@@ -31,6 +31,7 @@
 #include <QXmlStreamReader>
 
 #include <algorithm>
+#include <atomic>
 
 using namespace KSyntaxHighlighting;
 
@@ -42,11 +43,6 @@ DefinitionData::DefinitionData()
 
 DefinitionData::~DefinitionData() = default;
 
-DefinitionData *DefinitionData::get(const Definition &def)
-{
-    return def.d.get();
-}
-
 Definition::Definition()
     : d(new DefinitionData)
 {
@@ -91,7 +87,10 @@ QString Definition::name() const
 
 QString Definition::translatedName() const
 {
-    return QCoreApplication::instance()->translate("Language", d->name.toUtf8().constData());
+    if (d->translatedName.isEmpty()) {
+        d->translatedName = QCoreApplication::instance()->translate("Language", d->nameUtf8.isEmpty() ? d->name.toUtf8().constData() : d->nameUtf8.constData());
+    }
+    return d->translatedName;
 }
 
 QString Definition::section() const
@@ -101,15 +100,19 @@ QString Definition::section() const
 
 QString Definition::translatedSection() const
 {
-    return QCoreApplication::instance()->translate("Language Section", d->section.toUtf8().constData());
+    if (d->translatedSection.isEmpty()) {
+        d->translatedSection = QCoreApplication::instance()->translate("Language Section",
+                                                                       d->sectionUtf8.isEmpty() ? d->section.toUtf8().constData() : d->sectionUtf8.constData());
+    }
+    return d->translatedSection;
 }
 
-QVector<QString> Definition::mimeTypes() const
+QList<QString> Definition::mimeTypes() const
 {
     return d->mimetypes;
 }
 
-QVector<QString> Definition::extensions() const
+QList<QString> Definition::extensions() const
 {
     return d->extensions;
 }
@@ -217,12 +220,12 @@ bool Definition::setKeywordList(const QString &name, const QStringList &content)
     }
 }
 
-QVector<Format> Definition::formats() const
+QList<Format> Definition::formats() const
 {
     d->load();
 
     // sort formats so that the order matches the order of the itemDatas in the xml files.
-    auto formatList = QVector<Format>::fromList(d->formats.values());
+    auto formatList = QList<Format>::fromList(d->formats.values());
     std::sort(formatList.begin(), formatList.end(), [](const KSyntaxHighlighting::Format &lhs, const KSyntaxHighlighting::Format &rhs) {
         return lhs.id() < rhs.id();
     });
@@ -230,17 +233,17 @@ QVector<Format> Definition::formats() const
     return formatList;
 }
 
-QVector<Definition> Definition::includedDefinitions() const
+QList<Definition> Definition::includedDefinitions() const
 {
     d->load();
 
     // init worklist and result used as guard with this definition
-    QVector<const DefinitionData *> queue{d.get()};
-    QVector<Definition> definitions{*this};
+    QList<const DefinitionData *> queue{d.get()};
+    QList<Definition> definitions{*this};
     while (!queue.empty()) {
         const auto *def = queue.back();
         queue.pop_back();
-        for (const auto &defRef : def->immediateIncludedDefinitions) {
+        for (const auto &defRef : std::as_const(def->immediateIncludedDefinitions)) {
             const auto definition = defRef.definition();
             if (!definitions.contains(definition)) {
                 definitions.push_back(definition);
@@ -274,7 +277,7 @@ QPair<QString, QString> Definition::multiLineCommentMarker() const
     return {d->multiLineCommentStartMarker, d->multiLineCommentEndMarker};
 }
 
-QVector<QPair<QChar, QString>> Definition::characterEncodings() const
+QList<QPair<QChar, QString>> Definition::characterEncodings() const
 {
     d->load();
     return d->characterEncodings;
@@ -286,7 +289,7 @@ Context *DefinitionData::initialContext()
     return &contexts.front();
 }
 
-Context *DefinitionData::contextByName(const QString &wantedName)
+Context *DefinitionData::contextByName(QStringView wantedName)
 {
     for (auto &context : contexts) {
         if (context.name() == wantedName) {
@@ -317,6 +320,11 @@ bool DefinitionData::isLoaded() const
     return !contexts.empty();
 }
 
+namespace
+{
+std::atomic<uint64_t> definitionId{1};
+}
+
 bool DefinitionData::load(OnlyKeywords onlyKeywords)
 {
     if (fileName.isEmpty()) {
@@ -361,12 +369,15 @@ bool DefinitionData::load(OnlyKeywords onlyKeywords)
 
     resolveContexts();
 
+    id = definitionId.fetch_add(1, std::memory_order_relaxed);
+
     return true;
 }
 
 void DefinitionData::clear()
 {
     // keep only name and repo, so we can re-lookup to make references persist over repo reloads
+    id = 0;
     keywordLists.clear();
     contexts.clear();
     formats.clear();
@@ -385,7 +396,11 @@ void DefinitionData::clear()
     characterEncodings.clear();
 
     fileName.clear();
+    nameUtf8.clear();
+    translatedName.clear();
     section.clear();
+    sectionUtf8.clear();
+    translatedSection.clear();
     style.clear();
     indenter.clear();
     author.clear();
@@ -396,6 +411,9 @@ void DefinitionData::clear()
     version = 0.0f;
     priority = 0;
     hidden = false;
+
+    // purge our cache that is used to unify states
+    unify.clear();
 }
 
 bool DefinitionData::loadMetaData(const QString &definitionFileName)
@@ -424,7 +442,9 @@ bool DefinitionData::loadMetaData(const QString &definitionFileName)
 bool DefinitionData::loadMetaData(const QString &file, const QCborMap &obj)
 {
     name = obj.value(QLatin1String("name")).toString();
+    nameUtf8 = obj.value(QLatin1String("name")).toByteArray();
     section = obj.value(QLatin1String("section")).toString();
+    sectionUtf8 = obj.value(QLatin1String("section")).toByteArray();
     version = obj.value(QLatin1String("version")).toInteger();
     priority = obj.value(QLatin1String("priority")).toInteger();
     style = obj.value(QLatin1String("style")).toString();
@@ -435,13 +455,10 @@ bool DefinitionData::loadMetaData(const QString &file, const QCborMap &obj)
     fileName = file;
 
     const auto exts = obj.value(QLatin1String("extensions")).toString();
-    for (const auto &ext : exts.split(QLatin1Char(';'), Qt::SkipEmptyParts)) {
-        extensions.push_back(ext);
-    }
+    extensions = exts.split(QLatin1Char(';'), Qt::SkipEmptyParts);
+
     const auto mts = obj.value(QLatin1String("mimetype")).toString();
-    for (const auto &mt : mts.split(QLatin1Char(';'), Qt::SkipEmptyParts)) {
-        mimetypes.push_back(mt);
-    }
+    mimetypes = mts.split(QLatin1Char(';'), Qt::SkipEmptyParts);
 
     return true;
 }
@@ -465,13 +482,13 @@ bool DefinitionData::loadLanguage(QXmlStreamReader &reader)
     indenter = reader.attributes().value(QLatin1String("indenter")).toString();
     author = reader.attributes().value(QLatin1String("author")).toString();
     license = reader.attributes().value(QLatin1String("license")).toString();
-    const auto exts = reader.attributes().value(QLatin1String("extensions")).toString();
+    const auto exts = reader.attributes().value(QLatin1String("extensions"));
     for (const auto &ext : exts.split(QLatin1Char(';'), Qt::SkipEmptyParts)) {
-        extensions.push_back(ext);
+        extensions.push_back(ext.toString());
     }
-    const auto mts = reader.attributes().value(QLatin1String("mimetype")).toString();
+    const auto mts = reader.attributes().value(QLatin1String("mimetype"));
     for (const auto &mt : mts.split(QLatin1Char(';'), Qt::SkipEmptyParts)) {
-        mimetypes.push_back(mt);
+        mimetypes.push_back(mt.toString());
     }
     if (reader.attributes().hasAttribute(QLatin1String("casesensitive"))) {
         caseSensitive = Xml::attrToBool(reader.attributes().value(QLatin1String("casesensitive"))) ? Qt::CaseSensitive : Qt::CaseInsensitive;
@@ -592,15 +609,6 @@ void DefinitionData::resolveContexts()
     for (auto &context : contexts) {
         context.resolveIncludes(*this);
     }
-
-    /**
-     * Post-processing on rules.
-     */
-    for (const auto &context : contexts) {
-        for (auto &rule : context.rules()) {
-            rule->resolvePostProcessing();
-        }
-    }
 }
 
 void DefinitionData::loadItemData(QXmlStreamReader &reader)
@@ -614,7 +622,7 @@ void DefinitionData::loadItemData(QXmlStreamReader &reader)
             if (reader.name() == QLatin1String("itemData")) {
                 Format f;
                 auto formatData = FormatPrivate::detachAndGet(f);
-                formatData->definition = q;
+                formatData->definitionName = name;
                 formatData->load(reader);
                 formatData->id = RepositoryPrivate::get(repo)->nextFormatId();
                 formats.insert(f.name(), f);
@@ -707,7 +715,7 @@ void DefinitionData::loadComments(QXmlStreamReader &reader)
                 const bool isSingleLine = reader.attributes().value(QLatin1String("name")) == QLatin1String("singleLine");
                 if (isSingleLine) {
                     singleLineCommentMarker = reader.attributes().value(QLatin1String("start")).toString();
-                    const bool afterWhiteSpace = reader.attributes().value(QLatin1String("position")).toString() == QLatin1String("afterwhitespace");
+                    const bool afterWhiteSpace = reader.attributes().value(QLatin1String("position")) == QLatin1String("afterwhitespace");
                     singleLineCommentPosition = afterWhiteSpace ? CommentPosition::AfterWhitespace : CommentPosition::StartOfLine;
                 } else {
                     multiLineCommentStartMarker = reader.attributes().value(QLatin1String("start")).toString();
@@ -778,8 +786,8 @@ void DefinitionData::loadSpellchecking(QXmlStreamReader &reader)
             if (reader.name() == QLatin1String("encoding")) {
                 const auto charRef = reader.attributes().value(QLatin1String("char"));
                 if (!charRef.isEmpty()) {
-                    const auto str = reader.attributes().value(QLatin1String("string")).toString();
-                    characterEncodings.push_back({charRef[0], str});
+                    const auto str = reader.attributes().value(QLatin1String("string"));
+                    characterEncodings.push_back({charRef[0], str.toString()});
                 }
             }
             reader.readNext();
@@ -808,7 +816,7 @@ bool DefinitionData::checkKateVersion(QStringView verStr)
     const auto major = verStr.left(idx).toString().toInt();
     const auto minor = verStr.mid(idx + 1).toString().toInt();
 
-    if (major > SyntaxHighlighting_VERSION_MAJOR || (major == SyntaxHighlighting_VERSION_MAJOR && minor > SyntaxHighlighting_VERSION_MINOR)) {
+    if (major > KSYNTAXHIGHLIGHTING_VERSION_MAJOR || (major == KSYNTAXHIGHLIGHTING_VERSION_MAJOR && minor > KSYNTAXHIGHLIGHTING_VERSION_MINOR)) {
         qCWarning(Log) << "Skipping" << fileName << "due to being too new, version:" << verStr;
         return false;
     }
@@ -866,9 +874,12 @@ Definition DefinitionRef::definition() const
 
 bool DefinitionRef::operator==(const DefinitionRef &other) const
 {
-    if (d.expired() != other.d.expired()) {
-        return false;
-    }
-
-    return d.expired() || d.lock().get() == other.d.lock().get();
+    return !d.owner_before(other.d) && !other.d.owner_before(d);
 }
+
+bool DefinitionRef::operator==(const Definition &other) const
+{
+    return !d.owner_before(other.d) && !other.d.owner_before(d);
+}
+
+#include "moc_definition.cpp"
