@@ -7,7 +7,6 @@
 #ifndef KSYNTAXHIGHLIGHTING_FORMAT_P_H
 #define KSYNTAXHIGHLIGHTING_FORMAT_P_H
 
-#include "definitionref_p.h"
 #include "textstyledata_p.h"
 #include "theme.h"
 
@@ -22,6 +21,11 @@ public:
     FormatPrivate() = default;
     static FormatPrivate *detachAndGet(Format &format);
 
+    static std::intptr_t ptrId(const Format &format)
+    {
+        return std::intptr_t(format.d.data());
+    }
+
     TextStyleData styleOverride(const Theme &theme) const;
     void load(QXmlStreamReader &reader);
 
@@ -30,11 +34,11 @@ public:
     bool hasColor(const Theme &theme, StyleColor styleColor, ThemeColor themeColor) const;
     QColor color(const Theme &theme, StyleColor styleColor, ThemeColor themeColor) const;
 
-    DefinitionRef definition;
+    QString definitionName;
     QString name;
     TextStyleData style;
     Theme::TextStyle defaultStyle = Theme::Normal;
-    quint16 id = 0;
+    int id = 0;
     bool spellCheck = true;
 };
 
